@@ -28,9 +28,9 @@
 /*----------------------------------------------------------------------
               I n c l u d e s
 ----------------------------------------------------------------------*/
-#include "general.h"
-#include "tessclas.h"
-#include "varable.h"
+#include "host.h"
+#include "blobs.h"
+#include "params.h"
 
 /*----------------------------------------------------------------------
               T y p e s
@@ -38,10 +38,4 @@
 #define BASELINE_OFFSET 64
 #define BASELINE_SCALE  128
 
-/*----------------------------------------------------------------------
-              V a r i a b l e s
-----------------------------------------------------------------------*/
-
-extern INT_VAR_H(classify_baseline_normalized, 1, "Baseline Enable");
-
 #endif
