@@ -24,8 +24,7 @@
 /*----------------------------------------------------------------------
           Types
 ----------------------------------------------------------------------*/
-typedef struct sample
-{
+typedef struct sample {
   unsigned Clustered:1;          // TRUE if included in a higher cluster
   unsigned Prototype:1;          // TRUE if cluster represented by a proto
   unsigned SampleCount:30;       // number of samples in this cluster
@@ -33,22 +32,15 @@ typedef struct sample
   struct sample *Right;          // ptr to right sub-cluster
   inT32 CharID;                  // identifier of char sample came from
   FLOAT32 Mean[1];               // mean of cluster - SampleSize floats
-}
-
-
-CLUSTER;
+} CLUSTER;
 
 typedef CLUSTER SAMPLE;          // can refer to as either sample or cluster
 
 typedef enum {
   spherical, elliptical, mixed, automatic
-}
+} PROTOSTYLE;
 
-
-PROTOSTYLE;
-
-typedef struct                   // parameters to control clustering
-{
+typedef struct {                 // parameters to control clustering
   PROTOSTYLE ProtoStyle;         // specifies types of protos to be made
   FLOAT32 MinSamples;            // min # of samples per proto - % of total
   FLOAT32 MaxIllegal;            // max percentage of samples in a cluster which have
@@ -56,30 +48,18 @@ typedef struct                   // parameters to control clustering
   FLOAT32 Independence;          // desired independence between dimensions
   FLOAT64 Confidence;            // desired confidence in prototypes created
   int MagicSamples;              // Ideal number of samples in a cluster.
-}
-
-
-CLUSTERCONFIG;
+} CLUSTERCONFIG;
 
 typedef enum {
   normal, uniform, D_random
-}
+} DISTRIBUTION;
 
-
-DISTRIBUTION;
-
-typedef union
-{
+typedef union {
   FLOAT32 Spherical;
   FLOAT32 *Elliptical;
+} FLOATUNION;
 
-}
-
-
-FLOATUNION;
-
-typedef struct
-{
+typedef struct {
   unsigned Significant:1;        // TRUE if prototype is significant
   unsigned Merged:1;             // Merged after clustering so do not output
                                  // but kept for display purposes. If it has no
@@ -96,13 +76,9 @@ typedef struct
   FLOATUNION Variance;           // prototype variance
   FLOATUNION Magnitude;          // magnitude of density function
   FLOATUNION Weight;             // weight of density function
-}
+} PROTOTYPE;
 
-
-PROTOTYPE;
-
-typedef struct
-{
+typedef struct {
   inT16 SampleSize;              // number of parameters per sample
   PARAM_DESC *ParamDesc;         // description of each parameter
   inT32 NumberOfSamples;         // total number of samples being clustered
@@ -110,28 +86,22 @@ typedef struct
   CLUSTER *Root;                 // ptr to root cluster of cluster tree
   LIST ProtoList;                // list of prototypes
   inT32 NumChar;                 // # of characters represented by samples
-}
+  LIST bucket_cache[3];  // cache of reusable histograms by distribution type
+} CLUSTERER;
 
-
-CLUSTERER;
-
-typedef struct
-{
+typedef struct {
   inT32 NumSamples;              // number of samples in list
   inT32 MaxNumSamples;           // maximum size of list
   SAMPLE *Sample[1];             // array of ptrs to sample data structures
-}
-
-
-SAMPLELIST;
+} SAMPLELIST;
 
 // low level cluster tree analysis routines.
-#define InitSampleSearch(S,C) (((C)==NULL)?(S=NIL):(S=push(NIL,(C))))
+#define InitSampleSearch(S,C) (((C)==NULL)?(S=NIL_LIST):(S=push(NIL_LIST,(C))))
 
 /*--------------------------------------------------------------------------
         Public Function Prototypes
 --------------------------------------------------------------------------*/
-CLUSTERER *MakeClusterer (inT16 SampleSize, PARAM_DESC ParamDesc[]);
+CLUSTERER *MakeClusterer (inT16 SampleSize, const PARAM_DESC ParamDesc[]);
 
 SAMPLE *MakeSample (CLUSTERER * Clusterer, FLOAT32 Feature[], inT32 CharID);
 
@@ -141,7 +111,7 @@ void FreeClusterer(CLUSTERER *Clusterer);
 
 void FreeProtoList(LIST *ProtoList);
 
-void FreePrototype(void *arg);  //PROTOTYPE     *Prototype);
+void FreePrototype(void *arg);  // PROTOTYPE *Prototype);
 
 CLUSTER *NextSample(LIST *SearchState);
 
