@@ -28,8 +28,7 @@
 /*----------------------------------------------------------------------
               I n c l u d e s
 ----------------------------------------------------------------------*/
-#include "tessclas.h"
-#include "general.h"
+#include "blobs.h"
 
 /*----------------------------------------------------------------------
               M a c r o s
@@ -46,21 +45,6 @@
 #define is_inside_angle(pt)                                  \
 (angle_change ((pt)->prev, (pt), (pt)->next) < chop_inside_angle)
 
-/**********************************************************************
- * point_in_outline
- *
- * Check to see if this point falls within the bounding box of this
- * outline.  Note that this does not totally ensure that the edge
- * point falls on this outline.
- **********************************************************************/
-
-#define point_in_outline(p,o)         \
-((p)->pos.x >= (o)->topleft.x  &&   \
-	(p)->pos.y <= (o)->topleft.y  &&   \
-	(p)->pos.x <= (o)->botright.x &&   \
-	(p)->pos.y >= (o)->botright.y)     \
-
-
 /**********************************************************************
  * same_outline_bounds
  *
@@ -89,14 +73,10 @@
 /*----------------------------------------------------------------------
               F u n c t i o n s
 ----------------------------------------------------------------------*/
-void check_outline_mem(); 
-
 void correct_blob_order(TBLOB *blob1, TBLOB *blob2); 
 
 void eliminate_duplicate_outlines(TBLOB *blob); 
 
-void setup_outline(TESSLINE *outline); 
-
 void setup_blob_outlines(TBLOB *blob); 
 
 #endif
