@@ -1,8 +1,8 @@
 /**********************************************************************
  * File:        points.h  (Formerly coords.h)
  * Description: Coordinate class definitions.
- * Author:					Ray Smith
- * Created:					Fri Mar 15 08:32:45 GMT 1991
+ * Author:      Ray Smith
+ * Created:     Fri Mar 15 08:32:45 GMT 1991
  *
  * (C) Copyright 1991, Hewlett-Packard Ltd.
  ** Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,12 +23,11 @@
 #include          <stdio.h>
 #include          <math.h>
 #include          "elst.h"
-//#include                                      "ipeerr.h"
 
 class FCOORD;
 
 ///integer coordinate
-class DLLSYM ICOORD
+class ICOORD
 {
   friend class FCOORD;
 
@@ -37,33 +36,32 @@ class DLLSYM ICOORD
     ICOORD() {
       xcoord = ycoord = 0;       //default zero
     }
-	///constructor
-	///@param xin x value
-	///@param yin y value
+    ///constructor
+    ///@param xin x value
+    ///@param yin y value
     ICOORD(inT16 xin,
            inT16 yin) {
       xcoord = xin;
       ycoord = yin;
     }
-	///destructor
+    ///destructor
     ~ICOORD () {
     }
 
     ///access function
-    NEWDELETE2 (ICOORD) inT16 x () const
-    {
+    inT16 x() const {
       return xcoord;
     }
-	///access_function
+    ///access_function
     inT16 y() const {
       return ycoord;
     }
 
-	///rewrite function
+    ///rewrite function
     void set_x(inT16 xin) {
       xcoord = xin;              //write new value
     }
-	///rewrite function
+    ///rewrite function
     void set_y(inT16 yin) {  //value to set
       ycoord = yin;
     }
@@ -71,17 +69,17 @@ class DLLSYM ICOORD
     /// Set from the given x,y, shrinking the vector to fit if needed.
     void set_with_shrink(int x, int y);
 
-	///find sq length
+    ///find sq length
     float sqlength() const {
       return (float) (xcoord * xcoord + ycoord * ycoord);
     }
 
-	///find length
+    ///find length
     float length() const {
       return (float) sqrt (sqlength ());
     }
 
-	///sq dist between pts
+    ///sq dist between pts
     float pt_to_pt_sqdist(const ICOORD &pt) const {
       ICOORD gap;
 
@@ -90,55 +88,55 @@ class DLLSYM ICOORD
       return gap.sqlength ();
     }
 
-	///Distance between pts
+    ///Distance between pts
     float pt_to_pt_dist(const ICOORD &pt) const {
       return (float) sqrt (pt_to_pt_sqdist (pt));
     }
 
-	///find angle
+    ///find angle
     float angle() const {
       return (float) atan2 ((double) ycoord, (double) xcoord);
     }
 
-	///test equality
+    ///test equality
     BOOL8 operator== (const ICOORD & other) {
       return xcoord == other.xcoord && ycoord == other.ycoord;
     }
-	///test inequality
+    ///test inequality
     BOOL8 operator!= (const ICOORD & other) {
       return xcoord != other.xcoord || ycoord != other.ycoord;
     }
-	///rotate 90 deg anti
+    ///rotate 90 deg anti
     friend ICOORD operator! (const ICOORD &);
     ///unary minus
-	friend ICOORD operator- (const ICOORD &);
-	///add
-	friend ICOORD operator+ (const ICOORD &, const ICOORD &);
-	///add
+    friend ICOORD operator- (const ICOORD &);
+    ///add
+    friend ICOORD operator+ (const ICOORD &, const ICOORD &);
+    ///add
     friend ICOORD & operator+= (ICOORD &, const ICOORD &);
     ///subtract
-	friend ICOORD operator- (const ICOORD &, const ICOORD &);
+    friend ICOORD operator- (const ICOORD &, const ICOORD &);
     ///subtract
     friend ICOORD & operator-= (ICOORD &, const ICOORD &);
-	///scalar product
-	friend inT32 operator% (const ICOORD &, const ICOORD &);
-	///cross product
+    ///scalar product
+    friend inT32 operator% (const ICOORD &, const ICOORD &);
+    ///cross product
     friend inT32 operator *(const ICOORD &,
                             const ICOORD &);
-	///multiply
+    ///multiply
     friend ICOORD operator *(const ICOORD &,
                              inT16);
-	///multiply
+    ///multiply
     friend ICOORD operator *(inT16,
                              const ICOORD &);
-	///multiply
+    ///multiply
     friend ICOORD & operator*= (ICOORD &, inT16);
     ///divide
     friend ICOORD operator/ (const ICOORD &, inT16);
     ///divide
     friend ICOORD & operator/= (ICOORD &, inT16);
-	///rotate
-	///@param vec by vector
+    ///rotate
+    ///@param vec by vector
     void rotate(const FCOORD& vec);
 
     /// Setup for iterating over the pixels in a vector by the well-known
@@ -149,11 +147,6 @@ class DLLSYM ICOORD
     void setup_render(ICOORD* major_step, ICOORD* minor_step,
                       int* major, int* minor) const;
 
-	///serialise to ascii
-    void serialise_asc(FILE *f);
-	///serialise from ascii
-    void de_serialise_asc(FILE *f);
-
   protected:
     inT16 xcoord;                //< x value
     inT16 ycoord;                //< y value
@@ -166,59 +159,36 @@ class DLLSYM ICOORDELT:public ELIST_LINK, public ICOORD
     ///empty constructor
     ICOORDELT() {  
     }
-	///constructor from ICOORD
+    ///constructor from ICOORD
     ICOORDELT (ICOORD icoord):ICOORD (icoord) {
     }
-	///constructor
-	///@param xin x value
-	///@param yin y value
+    ///constructor
+    ///@param xin x value
+    ///@param yin y value
     ICOORDELT(inT16 xin,
               inT16 yin) {
       xcoord = xin;
       ycoord = yin;
     }
 
-    /* Note that prep_serialise() dump() and de_dump() dont need to do anything
-    more than terminate recursion. */
-
-	///set ptrs to counts
-    void prep_serialise() const {
-    }
-
-	///write external bits
-    void dump(FILE *) const {
-    }
-
-	///read external bits
-    void de_dump(FILE *) {
-    }
-
-    ///serialise to ascii
-    make_serialise(ICOORDELT)
-
     static ICOORDELT* deep_copy(const ICOORDELT* src) {
       ICOORDELT* elt = new ICOORDELT;
       *elt = *src;
       return elt;
     }
 
-	///serialise to ascii
-    void serialise_asc(FILE * f);
-	///deserialise from ascii
-    void de_serialise_asc(FILE *f);
-
 };
 
-ELISTIZEH_S (ICOORDELT)
+ELISTIZEH (ICOORDELT)
 class DLLSYM FCOORD
 {
   public:
     ///empty constructor
     FCOORD() {
     }
-	///constructor
-	///@param xvalue x value
-	///@param yvalue y value
+    ///constructor
+    ///@param xvalue x value
+    ///@param yvalue y value
     FCOORD(float xvalue,
            float yvalue) {
       xcoord = xvalue;           //set coords
@@ -236,26 +206,26 @@ class DLLSYM FCOORD
     float y() const {
       return ycoord;
     }
-	///rewrite function
+    ///rewrite function
     void set_x(float xin) {
       xcoord = xin;              //write new value
     }
-	///rewrite function
+    ///rewrite function
     void set_y(float yin) {  //value to set
       ycoord = yin;
     }
 
-	///find sq length
+    ///find sq length
     float sqlength() const {
       return xcoord * xcoord + ycoord * ycoord;
     }
 
-	///find length
+    ///find length
     float length() const {
       return (float) sqrt (sqlength ());
     }
 
-	///sq dist between pts
+    ///sq dist between pts
     float pt_to_pt_sqdist(const FCOORD &pt) const {
       FCOORD gap;
 
@@ -264,24 +234,24 @@ class DLLSYM FCOORD
       return gap.sqlength ();
     }
 
-	///Distance between pts
+    ///Distance between pts
     float pt_to_pt_dist(const FCOORD &pt) const {
       return (float) sqrt (pt_to_pt_sqdist (pt));
     }
 
-	///find angle
+    ///find angle
     float angle() const {
       return (float) atan2 (ycoord, xcoord);
     }
 
-	///Convert to unit vec
+    ///Convert to unit vec
     bool normalise();
 
-	///test equality
+    ///test equality
     BOOL8 operator== (const FCOORD & other) {
       return xcoord == other.xcoord && ycoord == other.ycoord;
     }
-	///test inequality
+    ///test inequality
     BOOL8 operator!= (const FCOORD & other) {
       return xcoord != other.xcoord || ycoord != other.ycoord;
     }
@@ -302,7 +272,7 @@ class DLLSYM FCOORD
     ///cross product
     friend float operator *(const FCOORD &, const FCOORD &);
     ///multiply
-	friend FCOORD operator *(const FCOORD &, float);
+    friend FCOORD operator *(const FCOORD &, float);
     ///multiply
     friend FCOORD operator *(float, const FCOORD &);
 
@@ -310,9 +280,12 @@ class DLLSYM FCOORD
     friend FCOORD & operator*= (FCOORD &, float);
     ///divide
     friend FCOORD operator/ (const FCOORD &, float);
-	///rotate
-	///@param vec by vector
+    ///rotate
+    ///@param vec by vector
     void rotate(const FCOORD vec);
+    // unrotate - undo a rotate(vec)
+    // @param vec by vector
+    void unrotate(const FCOORD &vec);
     ///divide
     friend FCOORD & operator/= (FCOORD &, float);
 
